#include "data_utility.h"

#include "operation.h"
#include "sphere.h"
#include "GeoGreedy.h"
#include "DMM.h"
#include "lp.h"


#ifndef WIN32
#include <sys/time.h>
#else
#include <ctime>
#endif

int main(int argc, char *argv[])
{
	if (argc != 3)
		return 0;

	char* input = argv[1];
	point_set_t* P = read_points(input);
	point_set_t* skyline = skyline_point(P);


	int k = atoi(argv[2]);
	double mrr, time;
	point_set_t* S;
	#ifndef WIN32
	struct timeval start, end;
	#else
	std::clock_t    start, end;
	#endif
		

	printf("------------k = %d------------\n", k);
	printf("%15s%15s%15s\n", "Algorithm", "mrr", "time");


	// Sphere
	#ifndef WIN32
	gettimeofday(&start, NULL);
	#else
	start = std::clock();
	#endif
	S = sphereWSImpLP(skyline, k);
	#ifndef WIN32
	gettimeofday(&end, NULL);
	time = (end.tv_sec - start.tv_sec) + (end.tv_usec - start.tv_usec) / 1000000.0;
	#else
	end = std::clock();
	time = (end - start) / (double)(CLOCKS_PER_SEC / 1000) / 1000;
	#endif
	mrr = evaluateLP(skyline, S, 0);
	//print_point_set(S);
	printf("%15s%15f%15f\n", "Sphere", mrr, time);
	release_point_set(S, false);
		


	// GeoGreedy
	#ifndef WIN32
	gettimeofday(&start, NULL);
	#else
	start = std::clock();
	#endif
	S = geoGreedy(k, skyline);
	#ifndef WIN32
	gettimeofday(&end, NULL);
	time = (end.tv_sec - start.tv_sec) + (end.tv_usec - start.tv_usec) / 1000000.0;
	#else
	end = std::clock();
	time = (end - start) / (double)(CLOCKS_PER_SEC / 1000) / 1000;
	#endif
	mrr = evaluateLP(skyline, S, 0);
	//print_point_set(S);
	printf("%15s%15f%15f\n", "GeoGreedy", mrr, time);
	release_point_set(S, false);

		
	// DMM-RRMS
	#ifndef WIN32
	gettimeofday(&start, NULL);
	#else
	start = std::clock();
	#endif
	S = DMM(skyline, k);
	#ifndef WIN32
	gettimeofday(&end, NULL);
	time = (end.tv_sec - start.tv_sec) + (end.tv_usec - start.tv_usec) / 1000000.0;
	#else
	end = std::clock();
	time = (end - start) / (double)(CLOCKS_PER_SEC / 1000) / 1000;
	#endif
	mrr = evaluateLP(skyline, S, 0);
	//print_point_set(S);
	printf("%15s%15f%15f\n", "DMM-RRMS", mrr, time);
	release_point_set(S, false);

	// DMM-Greedy
	#ifndef WIN32
	gettimeofday(&start, NULL);
	#else
	start = std::clock();
	#endif
	S = DMM_Greedy(skyline, k);
	#ifndef WIN32
	gettimeofday(&end, NULL);
	time = (end.tv_sec - start.tv_sec) + (end.tv_usec - start.tv_usec) / 1000000.0;
	#else
	end = std::clock();
	time = (end - start) / (double)(CLOCKS_PER_SEC / 1000) / 1000;
	#endif
	mrr = evaluateLP(skyline, S, 0);
	//print_point_set(S);
	printf("%15s%15f%15f\n", "DMM-Greedy", mrr, time);
	release_point_set(S, false);
	
	
	release_point_set(skyline, false);
	release_point_set(P, true);

	return 0;

}